﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Collections.ObjectModel;

namespace LinqTest
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            LinqTest();
        }

        private void LinqTest()
        {
            #region Data Insert
            PinMap pinmap1 = new PinMap() { pin_number = 3, pin_name = "XINPUT1" };
            PinMap pinmap2 = new PinMap() { pin_number = 8, pin_name = "XINPUT2" };
            PinMap pinmap3 = new PinMap() { pin_number = 11, pin_name = "XINPUT3" };
            PinMap pinmap4 = new PinMap() { pin_number = 24, pin_name = "XOUTPUT1" };
            PinMap pinmap5 = new PinMap() { pin_number = 59, pin_name = "XOUTPUT2" };

            Root root = new Root();
            root.listPinMap.Add(pinmap1);
            root.listPinMap.Add(pinmap2);
            root.listPinMap.Add(pinmap3);
            root.listPinMap.Add(pinmap4);
            root.listPinMap.Add(pinmap5);

            PinResult pResult = new PinResult();
            ushort t_idx1 = 8;
            pResult.pin_index.Add(t_idx1);
            float t_rslt1 = (float)3.14;
            pResult.pin_result.Add(t_rslt1);

            ushort t_idx2 = 24;
            pResult.pin_index.Add(t_idx2);
            float t_rslt2 = (float)5.87;
            pResult.pin_result.Add(t_rslt2);
            
            #endregion

            #region Query Test
            var query = from idx in pResult.pin_index
                        join pname in root.listPinMap
                            on idx equals pname.pin_number
                        select new { PIN_INDEX = pname.pin_number, PIN_NAME = pname.pin_name };

            dataGridView1.DataSource = query.ToList();

            // 질문입니다.
            // 위와 같이 PinResult로부터 Pin_index를 얻어서 PinMap.pin_name을 구했습니다.
            // 그런데 같은 Class 안에 있는 PinResult.pin_result도 같이 표시하고 싶은데
            // self join 이 안됩니다. 
            // PinResult.pin_index와 PinResult.pin_result는 ushort와 float로 서로 공통되는 부분이 없기 때문입니다.
            // 그냥 c style로 하면 PinResult.pin_index[i]와 pair되는 PinResult.pin_result[i]를 for로 돌리면 되는데
            // Linq의 장점도 못 살리고, Coding 량도 많아지게 됩니다.
            // 1. Class 설계를 어떤 식으로 개선해 볼 수 있을까요?
            // 2. Linq를 제가 너무 모르는가요?
            // 3. 따로 따로 구해서 Merge 하는게 좋을까요?
            // 전체적인 File 구조가 이런 식이라 대표적인 일례를 들어 여쭈어 봅니다.
            // 고맙습니다.
            #endregion
        }
    }

    public class Root
    {
        public Collection<PinMap> listPinMap { get; set; }

        public Root()
        {
            listPinMap = new Collection<PinMap>();
        }
    }

    public class PinResult
    {
        public Collection<ushort> pin_index { get; set; }
        public Collection<float> pin_result { get; set; }

        public PinResult()
        {
            pin_index = new Collection<ushort>();
            pin_result = new Collection<float>();
        }
    }

    public class PinMap
    {
        public ushort pin_number { get; set; }
        public string pin_name { get; set; }

        public PinMap()
        {
            pin_number = 0;
            pin_name = "";
        }
    }
}
